unit lptrelay;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, StdCtrls, DLPOrtIO, ExtCtrls ;

type
  Trelaymainform = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    lptportbase: TEdit;
    Help1: TMenuItem;
    About1: TMenuItem;
    Label1: TLabel;
    Port1: TPanel;
    Port2: TPanel;
    Port3: TPanel;
    Port4: TPanel;
    Port5: TPanel;
    Port6: TPanel;
    Port7: TPanel;
    Port8: TPanel;
    AllOff: TPanel;
    AllOn: TPanel;
    SetPortAddress: TButton;
    procedure Exit1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure lptportbaseExit(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Port1Click(Sender: TObject);
    procedure Port2Click(Sender: TObject);
    procedure Port3Click(Sender: TObject);
    procedure Port4Click(Sender: TObject);
    procedure Port5Click(Sender: TObject);
    procedure Port6Click(Sender: TObject);
    procedure Port7Click(Sender: TObject);
    procedure Port8Click(Sender: TObject);
    procedure AllOffClick(Sender: TObject);
    procedure AllOnClick(Sender: TObject);
    procedure SetPortAddressClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  relaymainform: Trelaymainform;
  vlptportbase,
  vnewlptportbase,
  vdataport,
  vstatusport,
  vcontrolport  : integer ;

implementation

{$R *.dfm}

procedure converttexttoint(st: string ; var num: integer ; limit: integer ; var cverror : boolean) ;

{ convert text string with number to integer, hex accepted, within limit
  if invalid then num := -1 }

begin


  cverror := false ;
 // We will catch conversion errors
  if st = ''
    then
      st := '0' ;
  try
    num := StrToInt(st);    // Trailing blanks are not supported

  except
   on Exception : EConvertError do
      begin
       ShowMessage('Not a valid number, hex or decimal expected');
       cverror := true ;
       end ;
  end;

  if not cverror  then
    begin
      if (num > limit) or (num < 0)
        then
          begin
            ShowMessage('Number out of range, maximum ' + inttostr(limit))  ;
            cverror := true ;
         end
    end ;


 end ; { ConvertTextToInt }

 procedure ReadIOPort(port : integer ; var value : integer) ;

  var

   lport : longint ;
   vport  : byte ;

 begin
   lport := port ;
   vport := DlPortReadPortUchar(lport) ;
   value := vport ;

 end ; { ReadIOPort }

 procedure WriteIOPort(port : integer ; value : integer) ;

  var

   lport : longint ;
   vport  : byte ;

 begin

   lport := port ;
   vport := value ;
   DlPortWritePortUchar(lport, vport) ;

 end ; { WriteIOPort }


procedure InitPort ;

begin

  { read controlport }
   ReadIOPort(vlptportbase + 2, vControlPort) ;


  { make sure it is set to not bidirectional }
  vcontrolport := vcontrolport or $DF ;
  WriteIOPort(vlptportbase + 2, vControlPort) ;



end ;



procedure Trelaymainform.Exit1Click(Sender: TObject);
begin

  halt ;

end;



procedure Trelaymainform.FormCreate(Sender: TObject);


begin

   vlptportbase := $378 ;
   vnewlptportbase := $378 ;
   lptportbase.text := '$378' ;
   { set to not bidirectional }
   InitPort ;
   { set all outputs off }
   vdataport := 0 ;

   Application.HelpFile := ExtractFilePath(Application.ExeName) + 'lptrelay.hlp';



end;

procedure Trelaymainform.lptportbaseExit(Sender: TObject);
var

  cverror : boolean ;
  num : integer ;

begin

  { convert to integer, range is byte }
   converttexttoint(lptportbase.text, num, $400, cverror) ;

  { check if lpt port addrsss}

  if not cverror
     then
       begin
         if not ((num = $378) or
            (num = $278) or
            (num = $3BC))
           then
             begin
               ShowMessage('Parallel port address is at $378, $278 or $3BC');
               lptportbase.text := '$378' ;
             end
           else
             begin
               vnewlptportbase := num ;
             end ;
       end ;

end;

procedure Trelaymainform.About1Click(Sender: TObject);
begin

 ShowMessage('LPT Relay V1.0, Hans Otten, (C) 2009, htp://www.hansotten.com') ;

end;

procedure Trelaymainform.Port1Click(Sender: TObject);
begin

  { change state and show it}
   if (vdataport and $01) = $01
     then
       begin
         vdataport := vdataport and $FE ;
         port1.Caption := 'Port 1 off' ;
         port1.color := clBtnFace ;
       end
     else
       begin
         vdataport := vdataport or $01 ;
         port1.Caption := 'Port 1 on';
         port1.color := clRed ;
       end ;
     WriteIOPort(vlptportbase, vDataPort);
     ReadIOPort(vlptportbase + 2, vControlPort) ;
     { send strobe to latch it in }
     vControlPort := vControlPort and $FE ;
     WriteIOPort(vlptportbase + 2, vControlPort);
     sleep(50) ;
     vControlPort := vControlPort or $01 ;
     WriteIOPort(vlptportbase + 2, vControlPort);

end ;

procedure Trelaymainform.Port2Click(Sender: TObject);
begin

  { change state and show it}
   if (vdataport and $02) = $02
     then
       begin
         vdataport := vdataport and $FD ;
         port2.Caption := 'Port 2 off' ;
         port2.color := clBtnFace ;
       end
     else
       begin
         vdataport := vdataport or $02 ;
         port2.Caption := 'Port 2 on';
         port2.color := clRed ;
       end ;
     WriteIOPort(vlptportbase, vDataPort);
     ReadIOPort(vlptportbase + 2, vControlPort) ;
     { send strobe to latch it in }
     vControlPort := vControlPort and $FE ;
     WriteIOPort(vlptportbase + 2, vControlPort);
     sleep(50) ;
     vControlPort := vControlPort or $01 ;
     WriteIOPort(vlptportbase + 2, vControlPort);




end;

procedure Trelaymainform.Port3Click(Sender: TObject);
begin

  { change state and show it}
   if (vdataport and $04) = $04
     then
       begin
         vdataport := vdataport and $FB    ;
         port3.Caption := 'Port 3 off' ;
         port3.color := clBtnFace ;
       end
     else
       begin
         vdataport := vdataport or $04 ;
         port3.Caption := 'Port 3 on';
         port3.color := clRed ;
       end ;
     WriteIOPort(vlptportbase, vDataPort);
     ReadIOPort(vlptportbase + 2, vControlPort) ;
     { send strobe to latch it in }
     vControlPort := vControlPort and $FE ;
     WriteIOPort(vlptportbase + 2, vControlPort);
     sleep(50) ;
     vControlPort := vControlPort or $01 ;
     WriteIOPort(vlptportbase + 2, vControlPort);



end;

procedure Trelaymainform.Port4Click(Sender: TObject);
begin
  { change state and show it}
   if (vdataport and $08) = $08
     then
       begin
         vdataport := vdataport and $F7 ;
         port4.Caption := 'Port 4 off' ;
         port4.color := clBtnFace ;
       end
     else
       begin
         vdataport := vdataport or $08 ;
         port4.Caption := 'Port 4 on';
         port4.color := clRed ;
       end ;
     WriteIOPort(vlptportbase, vDataPort);
     ReadIOPort(vlptportbase + 2, vControlPort) ;
     { send strobe to latch it in }
     vControlPort := vControlPort and $FE ;
     WriteIOPort(vlptportbase + 2, vControlPort);
     sleep(50) ;
     vControlPort := vControlPort or $01 ;
     WriteIOPort(vlptportbase + 2, vControlPort);


end;

procedure Trelaymainform.Port5Click(Sender: TObject);
begin
  { change state and show it}
   if (vdataport and $10) = $10
     then
       begin
         vdataport := vdataport and $EF ;
         port5.Caption := 'Port 5 off' ;
         port5.color := clBtnFace ;
       end
     else
       begin
         vdataport := vdataport or $10 ;
         port5.Caption := 'Port 5 on';
         port5.color := clRed ;
       end ;
     WriteIOPort(vlptportbase, vDataPort);
     ReadIOPort(vlptportbase + 2, vControlPort) ;
     { send strobe to latch it in }
     vControlPort := vControlPort and $FE ;
     WriteIOPort(vlptportbase + 2, vControlPort);
     sleep(50) ;
     vControlPort := vControlPort or $01 ;
     WriteIOPort(vlptportbase + 2, vControlPort);


end;

procedure Trelaymainform.Port6Click(Sender: TObject);
begin
  { change state and show it}
   if (vdataport and $20) = $20
     then
       begin
         vdataport := vdataport and $DF ;
         port6.Caption := 'Port 6 off' ;
         port6.color := clBtnFace ;
       end
     else
       begin
         vdataport := vdataport or $20 ;
         port6.Caption := 'Port 6 on';
         port6.color := clRed ;
       end ;
     WriteIOPort(vlptportbase, vDataPort);
     ReadIOPort(vlptportbase + 2, vControlPort) ;
     { send strobe to latch it in }
     vControlPort := vControlPort and $FE ;
     WriteIOPort(vlptportbase + 2, vControlPort);
     sleep(50) ;
     vControlPort := vControlPort or $01 ;
     WriteIOPort(vlptportbase + 2, vControlPort);


end;

procedure Trelaymainform.Port7Click(Sender: TObject);
begin
  { change state and show it}
   if (vdataport and $40) = $40
     then
       begin
         vdataport := vdataport and $BF ;
         port7.Caption := 'Port 7 off' ;
         port7.color := clBtnFace ;
       end
     else
       begin
         vdataport := vdataport or $40 ;
         port7.Caption := 'Port 7 on';
         port7.color := clRed ;
       end ;
     WriteIOPort(vlptportbase, vDataPort);
     ReadIOPort(vlptportbase + 2, vControlPort) ;
     { send strobe to latch it in }
     vControlPort := vControlPort and $FE ;
     WriteIOPort(vlptportbase + 2, vControlPort);
     sleep(50) ;
     vControlPort := vControlPort or $01 ;
     WriteIOPort(vlptportbase + 2, vControlPort);


end;

procedure Trelaymainform.Port8Click(Sender: TObject);
begin
  { change state and show it}
   if (vdataport and $80) = $80
     then
       begin
         vdataport := vdataport and $7F ;
         port8.Caption := 'Port 8 off' ;
         port8.color := clBtnFace ;
       end
     else
       begin
         vdataport := vdataport or $80 ;
         port8.Caption := 'Port 8 on';
         port8.color := clRed ;
       end ;
     WriteIOPort(vlptportbase, vDataPort);
     ReadIOPort(vlptportbase + 2, vControlPort) ;
     { send strobe to latch it in }
     vControlPort := vControlPort and $FE ;
     WriteIOPort(vlptportbase + 2, vControlPort);
     sleep(50) ;
     vControlPort := vControlPort or $01 ;
     WriteIOPort(vlptportbase + 2, vControlPort);


end;




procedure Trelaymainform.AllOffClick(Sender: TObject);
begin

     vDataPort := $00 ;
     WriteIOPort(vlptportbase, vDataPort);
     ReadIOPort(vlptportbase + 2, vControlPort) ;
     { send strobe to latch it in }
     vControlPort := vControlPort and $FE ;
     WriteIOPort(vlptportbase + 2, vControlPort);
     sleep(50) ;
     vControlPort := vControlPort or $01 ;
     WriteIOPort(vlptportbase + 2, vControlPort);

     port1.color := clBtnFace ;
     port1.Caption := 'Port 1 off' ;
     port2.color := clBtnFace;
     port2.Caption := 'Port 2 off' ;
     port3.color := clBtnFace;
     port3.Caption := 'Port 3 off' ;
     port4.color := clBtnFace;
     port4.Caption := 'Port 4 off' ;
     port5.color := clBtnFace;
     port5.Caption := 'Port 5 off' ;
     port6.color := clBtnFace;
     port6.Caption := 'Port 6 off' ;
     port7.color := clBtnFace ;
     port7.Caption := 'Port 7 off' ;
     port8.color := clBtnFace ;
     port8.Caption := 'Port 8 off' ;


end;

procedure Trelaymainform.AllOnClick(Sender: TObject);
begin

     vDataPort := $FF ;
     WriteIOPort(vlptportbase, vDataPort);
     ReadIOPort(vlptportbase + 2, vControlPort) ;
     { send strobe to latch it in }
     vControlPort := vControlPort and $FE ;
     WriteIOPort(vlptportbase + 2, vControlPort);
     sleep(50) ;
     vControlPort := vControlPort or $01 ;
     WriteIOPort(vlptportbase + 2, vControlPort);

     port1.color := clRed ;
     port1.Caption := 'Port 1 on' ;
     port2.color := clRed ;
     port2.Caption := 'Port 2 on' ;
     port3.color := clRed ;
     port3.Caption := 'Port 3 on' ;
     port4.color := clRed ;
     port4.Caption := 'Port 4 on' ;
     port5.color := clRed ;
     port5.Caption := 'Port 5 on' ;
     port6.color := clRed ;
     port6.Caption := 'Port 6 on' ;
     port7.color := clRed ;
     port7.Caption := 'Port 7 on' ;
     port8.color := clRed ;
     port8.Caption := 'Port 8 on' ;




end;

procedure Trelaymainform.SetPortAddressClick(Sender: TObject);

begin

  vlptportbase := vnewlptportbase ;

end;

end.
